@@ -4,7 +4,7 @@ import sys
 from contextlib import AbstractContextManager
 from copy import deepcopy
 from types import TracebackType
-from typing import TYPE_CHECKING, Any, Generic, TypeVar
+from typing import TYPE_CHECKING, Any
 
 from typing_extensions import Self
 
@@ -35,11 +35,13 @@ else:
 class CustomErrorMeta(type):
     """Custom base exception meta class."""
 
-    def __new__(cls: type[SelfCErrorMeta], *args: Any) -> SelfCErrorMeta:
+    def __new__[MetaSelf: CustomErrorMeta](cls: type[MetaSelf], *args: Any) -> MetaSelf:
         return CustomErrorMeta.setup_exception(super().__new__(cls, *args))
 
     @staticmethod
-    def setup_exception(exception: SelfCErrorMeta, override: str | ExceptionError | None = None) -> SelfCErrorMeta:
+    def setup_exception[MetaSelf: CustomErrorMeta](
+        exception: MetaSelf, override: str | ExceptionError | None = None
+    ) -> MetaSelf:
         """
         Setup an exception for later use in CustomError.
 
@@ -73,9 +75,6 @@ class CustomErrorMeta(type):
         return exception
 
 
-SelfCErrorMeta = TypeVar("SelfCErrorMeta", bound=CustomErrorMeta)
-
-
 class CustomError(ExceptionError, metaclass=CustomErrorMeta):
     """Custom base exception class."""
 
@@ -97,13 +96,6 @@ class CustomError(ExceptionError, metaclass=CustomErrorMeta):
 
         super().__init__(message)
 
-    def __class_getitem__(cls, exception: str | type[ExceptionError] | ExceptionError) -> CustomError:
-        from warnings import warn
-
-        warn("Custom error is not subscriptable anymore. Don't use it", DeprecationWarning)
-
-        return cls()
-
     def __call__(
         self,
         message: SupportsString | None | MissingT = MISSING,
@@ -170,20 +162,8 @@ class CustomError(ExceptionError, metaclass=CustomErrorMeta):
 
         out = f"{func_header}{self.message!s}{reason}".format(**self.kwargs).strip()
 
-        if sys.version_info < (3, 13) and hasattr(self, "__notes__"):
-            out += "\n" + "\n".join(self.__notes__)
-
         return out
 
-    if sys.version_info < (3, 11):
-        __notes__: list[str]
-
-        def add_note(self, note: str, /) -> None:
-            if not hasattr(self, "__notes__"):
-                self.__notes__ = []
-
-            self.__notes__.append(note)
-
     @classmethod
     def catch(cls) -> CatchError[Self]:
         """
@@ -196,20 +176,17 @@ class CustomError(ExceptionError, metaclass=CustomErrorMeta):
         return CatchError(cls)
 
 
-SelfError = TypeVar("SelfError", bound=CustomError)
-
-
-class CatchError(AbstractContextManager["CatchError[SelfError]"], Generic[SelfError]):
+class CatchError[CustomErrorT: CustomError](AbstractContextManager["CatchError[CustomErrorT]"]):
     """
     Context manager for catching a specific exception type.
     """
 
-    error: SelfError | None
+    error: CustomErrorT | None
     """The caught exception instance, if any."""
     tb: TracebackType | None
     """The traceback object associated with the caught exception."""
 
-    def __init__(self, error: type[SelfError]) -> None:
+    def __init__(self, error: type[CustomErrorT]) -> None:
         self.error = None
         self.tb = None
         self._to_catch_error = error
