@@ -1,11 +1,10 @@
 from __future__ import annotations
 
-import sys
 from fractions import Fraction
-from typing import Any, Callable, Iterable, Iterator, Sequence, overload
+from typing import Any, Callable, Iterable, Iterator, Protocol, Sequence, overload, runtime_checkable
 
 from ..exceptions import CustomOverflowError
-from ..types import SoftRange, SoftRangeN, SoftRangesN, StrictRange, SupportsString, T, is_soft_range_n
+from ..types import SoftRange, SoftRangeN, SoftRangesN, StrictRange, SupportsString, is_soft_range_n
 
 __all__ = [
     "flatten",
@@ -22,14 +21,14 @@ __all__ = [
 
 
 @overload
-def normalize_seq(val: T | Sequence[T], length: int) -> list[T]: ...
+def normalize_seq[T](val: T | Sequence[T], length: int) -> list[T]: ...
 
 
 @overload
 def normalize_seq(val: Any, length: int) -> list[Any]: ...
 
 
-def normalize_seq(val: T | Sequence[T], length: int) -> list[T]:
+def normalize_seq[T](val: T | Sequence[T], length: int) -> list[T]:
     """
     Normalize a sequence of values.
 
@@ -49,28 +48,23 @@ def normalize_seq(val: T | Sequence[T], length: int) -> list[T]:
 
 
 @overload
-def to_arr(val: T | Iterable[T]) -> list[T]: ...
+def to_arr[T](val: T | Iterable[T]) -> list[T]: ...
 
 
 @overload
 def to_arr(val: Any) -> list[Any]: ...
 
 
-def to_arr(val: Any, *, sub: Any = []) -> list[Any]:
+def to_arr(val: Any) -> list[Any]:
     """
     Normalize any value to a list.
     Bytes and str are not considered iterable and will not be flattened.
     """
-    if sub:
-        import warnings
-
-        warnings.warn("sub is deprecated.", DeprecationWarning)
-
     return list(val) if (isinstance(val, Iterable) and not isinstance(val, (str, bytes))) else [val]
 
 
 @overload
-def flatten(items: Iterable[Iterable[T]]) -> Iterator[T]: ...
+def flatten[T](items: Iterable[Iterable[T]]) -> Iterator[T]: ...
 
 
 @overload
@@ -207,7 +201,7 @@ def normalize_ranges(
         if any(
             [
                 start < 0 and end < 0,
-                start >= length and end - (not exclusive) > length,
+                start >= length and end + (not exclusive) > length,
                 start >= end + (not exclusive),
             ]
         ):
@@ -225,7 +219,7 @@ def normalize_ranges(
                 )
                 exceptions.append(exception)
                 continue
-            if end - (not exclusive) > length:
+            if end + (not exclusive) > length:
                 exception = CustomOverflowError(
                     f"End frame `{end}` in range `{r}` with length `{length}` could not be normalized!",
                     normalize_ranges,
@@ -239,10 +233,7 @@ def normalize_ranges(
         out.append((start, end))
 
     if exceptions:
-        if sys.version_info >= (3, 11):
-            raise ExceptionGroup("Multiple exceptions occurred!", exceptions)  # noqa: F821
-
-        raise Exception(exceptions)
+        raise ExceptionGroup("Multiple exceptions occurred!", exceptions)
 
     return normalize_list_to_ranges(
         [x for start, end in out for x in range(start, end + (not exclusive))], exclusive=exclusive
@@ -259,6 +250,13 @@ def invert_ranges(
     return normalize_list_to_ranges({*range(lengtha)} - b_frames, exclusive=exclusive)
 
 
+@runtime_checkable
+class _HasSelfAttr(Protocol):
+    __self__: type | object
+
+    def __call__(self, *args: Any, **kwds: Any) -> Any: ...
+
+
 def norm_func_name(func_name: SupportsString | Callable[..., Any]) -> str:
     """Normalize a class, function, or other object to obtain its name"""
 
@@ -275,7 +273,7 @@ def norm_func_name(func_name: SupportsString | Callable[..., Any]) -> str:
     elif hasattr(func_name, "__qualname__"):
         func_name = func.__qualname__
 
-    if callable(func) and hasattr(func, "__self__"):
+    if isinstance(func, _HasSelfAttr):
         func = func.__self__ if isinstance(func.__self__, type) else func.__self__.__class__
         func_name = f"{func.__name__}.{func_name}"
 
